/*
 *  Copyright 2019-2022 Diligent Graphics LLC
 *  Copyright 2015-2019 Egor Yusov
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  In no event and under no legal theory, whether in tort (including negligence),
 *  contract, or otherwise, unless required by applicable law (such as deliberate
 *  and grossly negligent acts) or agreed to in writing, shall any Contributor be
 *  liable for any damages, including any direct, indirect, special, incidental,
 *  or consequential damages of any character arising as a result of this License or
 *  out of the use or inability to use the software (including but not limited to damages
 *  for loss of goodwill, work stoppage, computer failure or malfunction, or any and
 *  all other commercial damages or losses), even if such Contributor has been advised
 *  of the possibility of such damages.
 */

#pragma once

/// \file
/// Declaration of Diligent::SamplerD3D12Impl class

#include "EngineD3D12ImplTraits.hpp"
#include "SamplerBase.hpp"
#include "DescriptorHeap.hpp"

namespace Diligent
{

/// Sampler object implementation in Direct3D12 backend.
class SamplerD3D12Impl final : public SamplerBase<EngineD3D12ImplTraits>
{
public:
    using TSamplerBase = SamplerBase<EngineD3D12ImplTraits>;

    SamplerD3D12Impl(IReferenceCounters*    pRefCounters,
                     RenderDeviceD3D12Impl* pRenderDeviceD3D12,
                     const SamplerDesc&     SamplerDesc);
    ~SamplerD3D12Impl();

    IMPLEMENT_QUERY_INTERFACE_IN_PLACE(IID_SamplerD3D12, TSamplerBase)

    /// Implementation of ISamplerD3D12::GetCPUDescriptorHandle().
    virtual D3D12_CPU_DESCRIPTOR_HANDLE DILIGENT_CALL_TYPE GetCPUDescriptorHandle() override { return m_Descriptor.GetCpuHandle(); }

private:
    friend class ShaderD3D12Impl;
    /// D3D12 sampler
    DescriptorHeapAllocation m_Descriptor;
};

} // namespace Diligent
